@@ -2,7 +2,7 @@
  *
  * Copyright (C) 2011-2016                                Inria
  * Copyright (C) 2013                                     Simon Archipoff
- * Copyright (C) 2008-2017                                Université de Bordeaux
+ * Copyright (C) 2008-2018                                Université de Bordeaux
  * Copyright (C) 2010-2015,2017                           CNRS
  * Copyright (C) 2013                                     Thibaut Lambert
  *
@@ -380,6 +380,7 @@ static int _starpu_push_task_on_specific_worker(struct starpu_task *task, int wo
 			struct starpu_task *alias = starpu_task_dup(task);
 			alias->destroy = 1;
 
+			_STARPU_TRACE_JOB_PUSH(alias, alias->priority > 0);
 			worker = _starpu_get_worker_struct(combined_workerid[j]);
 			ret |= _starpu_push_local_task(worker, alias, 0);
 		}
@@ -561,6 +562,7 @@ int _starpu_push_task_to_workers(struct starpu_task *task)
 				{
 					workerid = workers->get_next(workers, &it);
 					struct starpu_task *alias = starpu_task_dup(task);
+					_STARPU_TRACE_JOB_PUSH(alias, alias->priority > 0);
 					alias->destroy = 1;
 					ret |= _starpu_push_task_on_specific_worker(alias, workerid);
 				}
@@ -1105,3 +1107,8 @@ void _starpu_print_idle_time()
 		fclose(f);
 	}
 }
+
+void starpu_sched_task_break(struct starpu_task *task)
+{
+	_STARPU_TASK_BREAK_ON(task, sched);
+}
